import torch
from Code.util import to_gpu


"""
    Implements a candidate
"""


class Candidate(object):

    def __init__(self, policy_net, encoder, decoder, preference_vector, memory_dim=128):
        """
        :param policy_net: Instance of CandidatePolicy network
        :param encoder: Encoder network
        :param decoder: Decoder network
        :param preference_vector: (1, pref_dim) Preference vector for the candidate
        :param memory_dim: Dimension of memory vector used by the policy network
        """
        self.memory_dim = memory_dim

        # Initialize the networks
        self.policy_net = policy_net
        self.encoder = encoder
        self.decoder = decoder

        # Initialize the followers
        self.followers = []

        # Initialize the preference vector
        self.preference_vector = to_gpu(torch.from_numpy(preference_vector).float())

        # Initialize the memory
        self.memory = (to_gpu(torch.zeros(1, 1, self.memory_dim)), to_gpu(torch.zeros(1, 1, self.memory_dim)))

    def reset_followers(self):
        self.followers = []

    def add_follower(self, node):
        """
        :param node: Instance of node object that is a follower
        """
        self.followers.append(node)

    def broadcast(self, message):
        """
        :param message: (message_length, vocab_size) Representing the message
        """
        for follower in self.followers:
            follower.add_message(message)

    def take_action(self, adj_mat, current_voting, return_message=False):
        # Run the policy network
        memory, message = self.policy_net(self.memory, adj_mat, current_voting, self.preference_vector)

        # Update the memory
        self.memory = memory

        # Broadcast the message
        message = self.encoder(message)
        self.broadcast(message)

        if return_message:
            return torch.argmax(message, dim=1).tolist()