import os
import datetime


def print_config():
    print('Experiment Name: ' + EXP_NAME)
    print('Experiment Description: ' + EXP_DETAILS)
    print('Used GPU: ' + str(USE_GPU) + '\t' + str(CUDA_DEVICE))
    print('Preferred Candidate: ' + str(PREFERRED))
    print('Alternative Updates: ' + str(ALTERNATIVE_UPDATES))
    print('Loss Type: ' + LOSS_TYPE)
    print('Node Policy Edits Communicator: ' + str(NODE_EDIT_COMMUNICATOR))
    print('Candidate Policy Edits Communicator: ' + str(CANDIDATE_EDIT_COMMUNICATOR))
    print('Elementwise update of pref_vector: ' + str(ELEMENT_SIGMOID_PREF_UPDATE))
    print('Parameters:')
    print('\tLearning Rate: {0:3.5f}'.format(LEARNING_RATE))
    print('\tDisplay Step: {0:5.1f}'.format(DISPLAY_STEP))
    print('\tSave Step: {0:5.1f}'.format(SAVE_STEP))
    print('\tNumber of Episodes: {0:8.1f}'.format(NUM_EPISODES))
    print('\tEpisode Length: {0:3.1f}'.format(EPISODE_LENGTH))
    print('\tNumber of Nodes: {0:8.1f}'.format(NUM_NODES))
    print('\tMessage Dimension: {0:8.1f}'.format(MESSAGE_DIM))
    print('\tVocabulary Size: {0:8.1f}'.format(NUM_WORDS))
    print('\tMaximum Sentence Length: {0:8.1f}'.format(MAX_LENGTH))
    print('\tDimension of vocabulary words: {0:8.1f}'.format(EMBEDDING_DIM))
    print('\tDimension of Preference Vectors: {0:4.1f}'.format(PREF_DIM))
    print('\tDimension of additional memory for nodes: {0:4.1f}'.format(HIDDEN_DIM_NODE))
    print('\tDimension of memory for candidates: {0:4.1f}'.format(HIDDEN_DIM_CANDIDATE))
    print('\tInverse Temperature for Voting: {0:3.5f}'.format(INV_TEMP_VOTE))
    print('\tInverse Temperature for Following: {0:3.5f}'.format(INV_TEMP_FOLLOW))
    print('\tFollowing Temperature Learnable: ' + str(LEARN_FOLLOW_TEMP))


def read_config():
    global EXP_DETAILS, USE_GPU, CUDA_DEVICE, LOSS_TYPE, NODE_EDIT_COMMUNICATOR, \
           CANDIDATE_EDIT_COMMUNICATOR, ELEMENT_SIGMOID_PREF_UPDATE, \
           LEARNING_RATE, DISPLAY_STEP, SAVE_STEP, NUM_EPISODES, EPISODE_LENGTH, \
           NUM_NODES, MESSAGE_DIM, EMBEDDING_DIM, PREF_DIM, NUM_WORDS, MAX_LENGTH, \
           HIDDEN_DIM_NODE, HIDDEN_DIM_CANDIDATE, INV_TEMP_VOTE, INV_TEMP_FOLLOW, \
           LEARN_FOLLOW_TEMP, PREFERRED, ALTERNATIVE_UPDATES, ROOT_DIR

    ROOT_DIR = './' + EXP_NAME
    if os.path.exists(os.path.join(ROOT_DIR, 'config.txt')):
        with open(os.path.join(ROOT_DIR, 'config.txt')) as f:
            for line in f:
                if len(line.strip()) > 0:
                    tokens = line.strip().split(': ')
                    if 'Experiment Description' == tokens[0]:
                        EXP_DETAILS = tokens[1]
                    elif 'Used GPU' == tokens[0]:
                        temp = tokens[1].split('\t')
                        if temp[0] == 'True':
                            USE_GPU = True
                        else:
                            USE_GPU = False
                        CUDA_DEVICE = int(temp[1])
                    elif 'Preferred Candidate' == tokens[0]:
                        if tokens[1] == 'None':
                            PREFERRED = None
                        else:
                            PREFERRED = tokens[1]
                    elif 'Alternative Updates' == tokens[0]:
                        if tokens[1] == 'True':
                            ALTERNATIVE_UPDATES = True
                        else:
                            ALTERNATIVE_UPDATES = False
                    elif 'Loss Type' == tokens[0]:
                        LOSS_TYPE = tokens[1]
                    elif 'Node Policy Edits Communicator' == tokens[0]:
                        if tokens[1] == 'True':
                            NODE_EDIT_COMMUNICATOR = True
                        else:
                            NODE_EDIT_COMMUNICATOR = False
                    elif 'Candidate Policy Edits Communicator' == tokens[0]:
                        if tokens[1] == 'True':
                            CANDIDATE_EDIT_COMMUNICATOR = True
                        else:
                            CANDIDATE_EDIT_COMMUNICATOR = False
                    elif 'Elementwise update of pref_vector' == tokens[0]:
                        if tokens[1] == 'True':
                            ELEMENT_SIGMOID_PREF_UPDATE = True
                        else:
                            ELEMENT_SIGMOID_PREF_UPDATE = False
                    elif 'Learning Rate' == tokens[0]:
                        LEARNING_RATE = float(tokens[1].strip())
                    elif 'Display Step' == tokens[0]:
                        DISPLAY_STEP = int(float(tokens[1].strip()))
                    elif 'Save Step' == tokens[0]:
                        SAVE_STEP = int(float(tokens[1].strip()))
                    elif 'Number of Episodes' == tokens[0]:
                        NUM_EPISODES = int(float(tokens[1].strip()))
                    elif 'Episode Length' == tokens[0]:
                        EPISODE_LENGTH = int(float(tokens[1].strip()))
                    elif 'Number of Nodes' == tokens[0]:
                        NUM_NODES = int(float(tokens[1].strip()))
                    elif 'Message Dimension' == tokens[0]:
                        MESSAGE_DIM = int(float(tokens[1].strip()))
                    elif 'Vocabulary Size' == tokens[0]:
                        NUM_WORDS = int(float(tokens[1].strip()))
                    elif 'Maximum Sentence Length' == tokens[0]:
                        MAX_LENGTH = int(float(tokens[1].strip()))
                    elif 'Dimension of vocabulary words' == tokens[0]:
                        EMBEDDING_DIM = int(float(tokens[1].strip()))
                    elif 'Dimension of Preference Vectors' == tokens[0]:
                        PREF_DIM = int(float(tokens[1].strip()))
                    elif 'Dimension of additional memory for nodes' == tokens[0]:
                        HIDDEN_DIM_NODE = int(float(tokens[1].strip()))
                    elif 'Dimension of memory for candidates' == tokens[0]:
                        HIDDEN_DIM_CANDIDATE = int(float(tokens[1].strip()))
                    elif 'Dimension of memory for candidates' == tokens[0]:
                        HIDDEN_DIM_CANDIDATE = int(float(tokens[1].strip()))
                    elif 'Inverse Temperature for Voting' == tokens[0]:
                        INV_TEMP_VOTE = float(tokens[1].strip())
                    elif 'Inverse Temperature for Following' == tokens[0]:
                        INV_TEMP_FOLLOW = float(tokens[1].strip())
                    elif 'Following Temperature Learnable' == tokens[0]:
                        if tokens[1] == 'True':
                            LEARN_FOLLOW_TEMP = True
                        else:
                            LEARN_FOLLOW_TEMP = False
    else:
        print('Configuration not found.')
        exit(0)

    if USE_GPU:
        os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
        os.environ["CUDA_VISIBLE_DEVICES"] = str(CUDA_DEVICE)


def create_test_directories():
    global IMAGES_DIR, PLOTS_DIR, MODELS_DIR, LOG_FILE
    time = str(datetime.datetime.now())
    IMAGES_DIR = os.path.join(ROOT_DIR, 'images_test_' + time)
    os.mkdir(IMAGES_DIR)
    PLOTS_DIR = os.path.join(ROOT_DIR, 'plots_test_' + time)
    os.mkdir(PLOTS_DIR)
    MODELS_DIR = os.path.join(ROOT_DIR, 'saved_models')
    LOG_FILE = os.path.join(ROOT_DIR, 'log_test_' + time + '.txt')


# Configuration options
USE_GPU = True
CUDA_DEVICE = 0

LEARNING_RATE = 1e-3

DISPLAY_STEP = 20
SAVE_STEP = 500

EXP_NAME = 'Exp_' + str(datetime.datetime.now())
EXP_DETAILS = ''
ROOT_DIR = './' + EXP_NAME
MODELS_DIR = os.path.join(ROOT_DIR, 'saved_models')
IMAGES_DIR = os.path.join(ROOT_DIR, 'images')
PLOTS_DIR = os.path.join(ROOT_DIR, 'plots')
LOG_FILE = os.path.join(ROOT_DIR, 'log.txt')

NUM_EPISODES = 10000
EPISODE_LENGTH = 5

NUM_NODES = 100

MESSAGE_DIM = 16
EMBEDDING_DIM = 16
PREF_DIM = 2
NUM_WORDS = 1024
MAX_LENGTH = 5
HIDDEN_DIM_NODE = 16
HIDDEN_DIM_CANDIDATE = 16

INV_TEMP_VOTE = 2.0
INV_TEMP_FOLLOW = 2.0
LEARN_FOLLOW_TEMP = False

LOSS_TYPE = 'SheepLoss'

PREFERRED = 'C1'
ALTERNATIVE_UPDATES = True

NODE_EDIT_COMMUNICATOR = True
CANDIDATE_EDIT_COMMUNICATOR = True

ELEMENT_SIGMOID_PREF_UPDATE = False

RESTORE_FROM_CONFIG = False
MODEL_NUM = 10000


if USE_GPU:
    os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
    os.environ["CUDA_VISIBLE_DEVICES"] = str(CUDA_DEVICE)


if not RESTORE_FROM_CONFIG:
    os.mkdir('./' + EXP_NAME)
    os.mkdir(MODELS_DIR)
    os.mkdir(IMAGES_DIR)
    os.mkdir(PLOTS_DIR)
