import torch
import torch.nn as nn
import Code.util as util
from torch.distributions import Gumbel


"""
    Implements the Gumbel Softmax module
"""


class GumbelSoftmax(nn.Module):
    def __init__(self):
        """
        :param temp: Temperature to be used
        """
        super(GumbelSoftmax, self).__init__()
        self.gumbel = Gumbel(torch.tensor([0.0]), torch.tensor([1.0]))
        self.softmax = nn.Softmax(dim=1)

    def forward(self, x, inv_temp=1):
        """
        :param x: Input on which Gumbel Softmax has to be applied
        :param inv_temp: Inverse of the temperature
        :return:
            y: Same size as x
        """
        y = x + util.to_gpu(self.gumbel.sample(x.size()).squeeze(2))
        return self.softmax(y * inv_temp)

    def create_one_hot(self, x):
        """
        :param x: Gumbel softmax output
        :return:
            x_hat: One hot of the same size as x
        """
        idx = torch.max(x, dim=1)[1].view(-1, 1)
        one_hot = torch.zeros_like(x)
        one_hot.scatter_(1, idx, 1)
        one_hot = (one_hot - x).detach() + x
        return one_hot
