import torch
import torch.nn as nn
from Code.GumbelSoftmax import GumbelSoftmax


"""
    Implements the Voting policy network.
"""


class Voting(nn.Module):
    def __init__(self):
        """
        :param init_param: Initial value  of parameter used for computing inverse temperature
        """
        super(Voting, self).__init__()
        self.gumbel_softmax_vote = GumbelSoftmax()

    def forward(self, distances, inv_temp):
        """
        :param distances: (1+, 2) Distances to various candidates
        :param inv_temp: Inverse temperature to be used during voting
        :return:
            voting: (1+, 2) One hot vector representing voting
        """
        return self.gumbel_softmax_vote.create_one_hot(self.gumbel_softmax_vote(-distances, inv_temp))
